@@ -25,6 +25,12 @@
  * SUCH DAMAGE.
  */
 
+#include <sys/socket.h>
+
+#include <net/if.h>
+#include <netinet/in.h>
+#include <netinet/if_ether.h>
+
 #include <errno.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -39,14 +45,14 @@
 #include "dhcp.h"
 #include "dhcpcd.h"
 #include "eloop.h"
+#include "if.h"
 #include "if-options.h"
 #include "ipv4ll.h"
-#include "net.h"
 
 #define ARP_LEN								      \
 	(sizeof(struct arphdr) + (2 * sizeof(uint32_t)) + (2 * HWADDR_LEN))
 
-static int
+static ssize_t
 arp_send(const struct interface *ifp, int op, in_addr_t sip, in_addr_t tip)
 {
 	uint8_t arp_buffer[ARP_LEN];
@@ -61,15 +67,15 @@ arp_send(const struct interface *ifp, int op, in_addr_t sip, in_addr_t tip)
 	ar.ar_op = htons(op);
 
 	p = arp_buffer;
-	len = sizeof(arp_buffer);
+	len = 0;
 
 #define CHECK(fun, b, l)						\
 	do {								\
-		if (len < (l))						\
+		if (len + (l) > sizeof(arp_buffer))			\
 			goto eexit;					\
 		fun(p, (b), (l));					\
 		p += (l);						\
-		len -= (l);						\
+		len += (l);						\
 	} while (/* CONSTCOND */ 0)
 #define APPEND(b, l)	CHECK(memcpy, b, l)
 #define ZERO(l)		CHECK(memset, 0, l)
@@ -79,8 +85,7 @@ arp_send(const struct interface *ifp, int op, in_addr_t sip, in_addr_t tip)
 	APPEND(&sip, sizeof(sip));
 	ZERO(ifp->hwlen);
 	APPEND(&tip, sizeof(tip));
-	len = p - arp_buffer;
-	return ipv4_sendrawpacket(ifp, ETHERTYPE_ARP, arp_buffer, len);
+	return if_sendrawpacket(ifp, ETHERTYPE_ARP, arp_buffer, len);
 
 eexit:
 	errno = ENOBUFS;
@@ -108,10 +113,10 @@ arp_failure(struct interface *ifp)
 		dhcp_decline(ifp);
 	eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
 	if (state->lease.frominfo)
-		start_interface(ifp);
+		dhcpcd_startinterface(ifp);
 	else
 		eloop_timeout_add_sec(ifp->ctx->eloop,
-		    DHCP_ARP_FAIL, start_interface, ifp);
+		    DHCP_ARP_FAIL, dhcpcd_startinterface, ifp);
 }
 
 static void
@@ -129,14 +134,20 @@ arp_packet(void *arg)
 	const char *hwaddr;
 	struct in_addr ina;
 	char hwbuf[HWADDR_LEN * 3];
+	int flags;
 
 	state = D_STATE(ifp);
 	state->fail.s_addr = 0;
-	for(;;) {
-		bytes = ipv4_getrawpacket(ifp, ETHERTYPE_ARP,
-		    arp_buffer, sizeof(arp_buffer), NULL);
-		if (bytes == 0 || bytes == -1)
+	flags = 0;
+	while (!(flags & RAW_EOF)) {
+		bytes = if_readrawpacket(ifp, ETHERTYPE_ARP,
+		    arp_buffer, sizeof(arp_buffer), &flags);
+		if (bytes == 0 || bytes == -1) {
+			syslog(LOG_ERR, "%s: arp if_readrawpacket: %m",
+			    ifp->name);
+			dhcp_close(ifp);
 			return;
+		}
 		/* We must have a full ARP header */
 		if ((size_t)bytes < sizeof(ar))
 			continue;
@@ -178,8 +189,8 @@ arp_packet(void *arg)
 			syslog(LOG_INFO,
 			    "%s: found %s on hardware address %s",
 			    ifp->name, inet_ntoa(ina), hwaddr);
-			if (select_profile(ifp, hwaddr) == -1 &&
-			    select_profile(ifp, inet_ntoa(ina)) == -1)
+			if (dhcpcd_selectprofile(ifp, hwaddr) == -1 &&
+			    dhcpcd_selectprofile(ifp, inet_ntoa(ina)) == -1)
 			{
 				state->probes = 0;
 				/* We didn't find a profile for this
@@ -194,7 +205,7 @@ arp_packet(void *arg)
 			}
 			dhcp_close(ifp);
 			eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
-			start_interface(ifp);
+			dhcpcd_startinterface(ifp);
 			return;
 		}
 
@@ -233,7 +244,7 @@ arp_announce(void *arg)
 	if (state->new == NULL)
 		return;
 	if (state->arp_fd == -1) {
-		state->arp_fd = ipv4_opensocket(ifp, ETHERTYPE_ARP);
+		state->arp_fd = if_openrawsocket(ifp, ETHERTYPE_ARP);
 		if (state->arp_fd == -1) {
 			syslog(LOG_ERR, "%s: %s: %m", __func__, ifp->name);
 			return;
@@ -269,7 +280,8 @@ arp_announce(void *arg)
 		state->probes = 0;
 		state->claims = 0;
 		tv.tv_sec = state->interval - DHCP_RAND_MIN;
-		tv.tv_usec = arc4random() % (DHCP_RAND_MAX_U - DHCP_RAND_MIN_U);
+		tv.tv_usec = (suseconds_t)arc4random_uniform(
+		    (DHCP_RAND_MAX - DHCP_RAND_MIN) * 1000000);
 		timernorm(&tv);
 		eloop_timeout_add_tv(ifp->ctx->eloop, &tv, dhcp_discover, ifp);
 	} else {
@@ -289,7 +301,7 @@ arp_probe(void *arg)
 	int arping = 0;
 
 	if (state->arp_fd == -1) {
-		state->arp_fd = ipv4_opensocket(ifp, ETHERTYPE_ARP);
+		state->arp_fd = if_openrawsocket(ifp, ETHERTYPE_ARP);
 		if (state->arp_fd == -1) {
 			syslog(LOG_ERR, "%s: %s: %m", __func__, ifp->name);
 			return;
@@ -319,7 +331,8 @@ arp_probe(void *arg)
 	}
 	if (++state->probes < PROBE_NUM) {
 		tv.tv_sec = PROBE_MIN;
-		tv.tv_usec = arc4random() % (PROBE_MAX_U - PROBE_MIN_U);
+		tv.tv_usec = (suseconds_t)arc4random_uniform(
+		    (PROBE_MAX - PROBE_MIN) * 1000000);
 		timernorm(&tv);
 		eloop_timeout_add_tv(ifp->ctx->eloop, &tv, arp_probe, ifp);
 	} else {
@@ -332,7 +345,7 @@ arp_probe(void *arg)
 				    &tv, arp_probe, ifp);
 			else
 				eloop_timeout_add_tv(ifp->ctx->eloop,
-				    &tv, start_interface, ifp);
+				    &tv, dhcpcd_startinterface, ifp);
 		} else
 			eloop_timeout_add_tv(ifp->ctx->eloop,
 			    &tv, dhcp_bind, ifp);
