@@ -93,6 +93,7 @@
 #define O_CONTROLGRP		O_BASE + 34
 #define O_SLAAC			O_BASE + 35
 #define O_GATEWAY		O_BASE + 36
+#define O_PFXDLGMIX		O_BASE + 37
 
 const struct option cf_options[] = {
 	{"background",      no_argument,       NULL, 'b'},
@@ -179,6 +180,7 @@ const struct option cf_options[] = {
 	{"controlgroup",    required_argument, NULL, O_CONTROLGRP},
 	{"slaac",           required_argument, NULL, O_SLAAC},
 	{"gateway",         no_argument,       NULL, O_GATEWAY},
+	{"ia_pd_mix",       no_argument,       NULL, O_PFXDLGMIX},
 	{NULL,              0,                 NULL, '\0'}
 };
 
@@ -518,7 +520,9 @@ parse_addr(__unused struct in_addr *addr, __unused struct in_addr *net,
 
 static const char *
 set_option_space(struct dhcpcd_ctx *ctx,
-    const char *arg, const struct dhcp_opt **d, size_t *dl,
+    const char *arg,
+    const struct dhcp_opt **d, size_t *dl,
+    const struct dhcp_opt **od, size_t *odl,
     struct if_options *ifo,
     uint8_t *request[], uint8_t *require[], uint8_t *no[])
 {
@@ -527,6 +531,8 @@ set_option_space(struct dhcpcd_ctx *ctx,
 	if (strncmp(arg, "dhcp6_", strlen("dhcp6_")) == 0) {
 		*d = ctx->dhcp6_opts;
 		*dl = ctx->dhcp6_opts_len;
+		*od = ifo->dhcp6_override;
+		*odl = ifo->dhcp6_override_len;
 		*request = ifo->requestmask6;
 		*require = ifo->requiremask6;
 		*no = ifo->nomask6;
@@ -537,9 +543,13 @@ set_option_space(struct dhcpcd_ctx *ctx,
 #ifdef INET
 	*d = ctx->dhcp_opts;
 	*dl = ctx->dhcp_opts_len;
+	*od = ifo->dhcp_override;
+	*odl = ifo->dhcp_override_len;
 #else
 	*d = NULL;
 	*dl = 0;
+	*od = NULL;
+	*odl = 0;
 #endif
 	*request = ifo->requestmask;
 	*require = ifo->requiremask;
@@ -630,15 +640,14 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 	struct in_addr addr, addr2;
 	in_addr_t *naddr;
 	struct rt *rt;
-	const struct dhcp_opt *d;
+	const struct dhcp_opt *d, *od;
 	uint8_t *request, *require, *no;
 	struct dhcp_opt **dop, *ndop;
-	size_t *dop_len, dl;
+	size_t *dop_len, dl, odl;
 	struct vivco *vivco;
 	struct token *token;
 	struct group *grp;
 #ifdef _REENTRANT
-#error foo
 	struct group grpbuf;
 #endif
 #ifdef INET6
@@ -733,10 +742,10 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case 'o':
-		arg = set_option_space(ctx, arg, &d, &dl, ifo,
+		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no);
-		if (make_option_mask(d, dl, request, arg, 1) != 0 ||
-		    make_option_mask(d, dl, no, arg, -1) != 0)
+		if (make_option_mask(d, dl, od, odl, request, arg, 1) != 0 ||
+		    make_option_mask(d, dl, od, odl, no, arg, -1) != 0)
 		{
 			syslog(LOG_ERR, "unknown option `%s'", arg);
 			return -1;
@@ -952,22 +961,22 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		ifo->options |= DHCPCD_MASTER;
 		break;
 	case 'O':
-		arg = set_option_space(ctx, arg, &d, &dl, ifo,
+		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no);
-		if (make_option_mask(d, dl, request, arg, -1) != 0 ||
-		    make_option_mask(d, dl, require, arg, -1) != 0 ||
-		    make_option_mask(d, dl, no, arg, 1) != 0)
+		if (make_option_mask(d, dl, od, odl, request, arg, -1) != 0 ||
+		    make_option_mask(d, dl, od, odl, require, arg, -1) != 0 ||
+		    make_option_mask(d, dl, od, odl, no, arg, 1) != 0)
 		{
 			syslog(LOG_ERR, "unknown option `%s'", arg);
 			return -1;
 		}
 		break;
 	case 'Q':
-		arg = set_option_space(ctx, arg, &d, &dl, ifo,
+		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
 		    &request, &require, &no);
-		if (make_option_mask(d, dl, require, arg, 1) != 0 ||
-		    make_option_mask(d, dl, request, arg, 1) != 0 ||
-		    make_option_mask(d, dl, no, arg, -1) != 0)
+		if (make_option_mask(d, dl, od, odl, require, arg, 1) != 0 ||
+		    make_option_mask(d, dl, od, odl, request, arg, 1) != 0 ||
+		    make_option_mask(d, dl, od, odl, no, arg, -1) != 0)
 		{
 			syslog(LOG_ERR, "unknown option `%s'", arg);
 			return -1;
@@ -1156,8 +1165,11 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		}
 		break;
 	case O_DESTINATION:
-		if (make_option_mask(ctx->dhcp_opts, ctx->dhcp_opts_len,
-		    ifo->dstmask, arg, 2) != 0) {
+		arg = set_option_space(ctx, arg, &d, &dl, &od, &odl, ifo,
+		    &request, &require, &no);
+		if (make_option_mask(d, dl, od, odl,
+		    ifo->dstmask, arg, 2) != 0)
+		{
 			if (errno == EINVAL)
 				syslog(LOG_ERR, "option `%s' does not take"
 				    " an IPv4 address", arg);
@@ -1220,35 +1232,41 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			}
 			i = D6_OPTION_IA_PD;
 		}
-		if (arg != NULL && ifname == NULL) {
+		if (ifname == NULL && arg) {
 			syslog(LOG_ERR,
 			    "IA with IAID must belong in an interface block");
 			return -1;
 		}
 		ifo->options |= DHCPCD_IA_FORCED;
-		if (ifo->ia_type != 0 && ifo->ia_type != i) {
-			syslog(LOG_ERR, "cannot specify a different IA type");
-			return -1;
-		}
-		ifo->ia_type = (uint16_t)i;
-		if (arg == NULL)
-			break;
 		fp = strwhite(arg);
-		if (fp)
+		if (fp) {
 			*fp++ = '\0';
-		if (parse_iaid(iaid, arg, sizeof(iaid)) == -1)
-			return -1;
+			fp = strskipwhite(fp);
+		}
+		if (arg) {
+			p = strchr(arg, '/');
+			if (p)
+				*p++ = '\0';
+			if (parse_iaid(iaid, arg, sizeof(iaid)) == -1)
+				return -1;
+		}
 		ia = NULL;
 		for (sl = 0; sl < ifo->ia_len; sl++) {
-			if (ifo->ia[sl].iaid[0] == iaid[0] &&
+			if ((arg == NULL && !ifo->ia[sl].iaid_set) ||
+			    (ifo->ia[sl].iaid_set &&
+			    ifo->ia[sl].iaid[0] == iaid[0] &&
 			    ifo->ia[sl].iaid[1] == iaid[1] &&
 			    ifo->ia[sl].iaid[2] == iaid[2] &&
-			    ifo->ia[sl].iaid[3] == iaid[3])
+			    ifo->ia[sl].iaid[3] == iaid[3]))
 			{
 			        ia = &ifo->ia[sl];
 				break;
 			}
 		}
+		if (ia && ia->ia_type != (uint16_t)i) {
+			syslog(LOG_ERR, "Cannot mix IA for the same IAID");
+			break;
+		}
 		if (ia == NULL) {
 			ia = realloc(ifo->ia,
 			    sizeof(*ifo->ia) * (ifo->ia_len + 1));
@@ -1258,14 +1276,47 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			}
 			ifo->ia = ia;
 			ia = &ifo->ia[ifo->ia_len++];
-			ia->iaid[0] = iaid[0];
-			ia->iaid[1] = iaid[1];
-			ia->iaid[2] = iaid[2];
-			ia->iaid[3] = iaid[3];
-			ia->sla = NULL;
+			ia->ia_type = (uint16_t)i;
+			if (arg) {
+				ia->iaid[0] = iaid[0];
+				ia->iaid[1] = iaid[1];
+				ia->iaid[2] = iaid[2];
+				ia->iaid[3] = iaid[3];
+				ia->iaid_set = 1;
+			} else
+				ia->iaid_set = 0;
+			if (!ia->iaid_set ||
+			    p == NULL ||
+			    ia->ia_type == D6_OPTION_IA_TA)
+			{
+				memset(&ia->addr, 0, sizeof(ia->addr));
+				ia->prefix_len = 0;
+			} else {
+				arg = p;
+				p = strchr(arg, '/');
+				if (p)
+					*p++ = '\0';
+				if (inet_pton(AF_INET6, arg, &ia->addr) == -1) {
+					syslog(LOG_ERR, "%s: %m", arg);
+					memset(&ia->addr, 0, sizeof(ia->addr));
+				}
+				if (p && ia->ia_type == D6_OPTION_IA_PD) {
+					i = atoint(p);
+					if (i != -1 && (i < 8 || i > 120)) {
+						errno = EINVAL;
+						i = -1;
+					}
+					if (i == -1) {
+						syslog(LOG_ERR, "%s: %m", p);
+						ia->prefix_len = 0;
+					} else
+						ia->prefix_len = (uint8_t)i;
+				}
+			}
 			ia->sla_len = 0;
+			ia->sla = NULL;
 		}
-		if (ifo->ia_type != D6_OPTION_IA_PD)
+		if (ia->ia_type != D6_OPTION_IA_PD)
 			break;
 		for (p = fp; p; p = fp) {
 			fp = strwhite(p);
@@ -1284,12 +1335,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			np = strchr(p, '/');
 			if (np)
 				*np++ = '\0';
-			if (strcmp(ifname, p) == 0) {
-				syslog(LOG_ERR,
-				    "%s: cannot assign IA_PD to itself",
-				    ifname);
-				goto err_sla;
-			}
 			if (strlcpy(sla->ifname, p,
 			    sizeof(sla->ifname)) >= sizeof(sla->ifname))
 			{
@@ -1853,6 +1898,9 @@ err_sla:
 		else
 			ifo->options &= ~DHCPCD_SLAACPRIVATE;
 		break;
+	case O_PFXDLGMIX:
+		ifo->options |= DHCPCD_PFXDLGMIX;
+		break;
 	default:
 		return 0;
 	}
@@ -2124,6 +2172,10 @@ read_config(struct dhcpcd_ctx *ctx,
 				skip = 1;
 			continue;
 		}
+		/* Skip arping if we have selected a profile but not parsing
+		 * one. */
+		if (profile && !have_profile && strcmp(option, "arping") == 0)
+			continue;
 		if (skip)
 			continue;
 		parse_config_line(ctx, ifname, ifo, option, line, &ldop, &edop);
